/******************************************************************************
* Include files
******************************************************************************/
#include <avr/io.h>
#include "rtc.h"

/******************************************************************************
* Macros
******************************************************************************/

#define SLA_W	0b11010000	// slave write
#define SLA_R	0b11010001	// slave read

/******************************************************************************
* Constants
******************************************************************************/


/******************************************************************************
* Global Variables
******************************************************************************/



/******************************************************************************
* External Variables
******************************************************************************/
volatile uint8_t Task_1000ms,Task_100ms;
date_t date;

/******************************************************************************
* Local Function Declarations
******************************************************************************/

/******************************************************************************
* Local Function Definitions
******************************************************************************/

/******************************************************************************
* Global Function Definitions
******************************************************************************/

/******************************************************************************
* Function:         	void TWRC_start(void)
* Description:		
* Input:
* Output:
* Notes:
******************************************************************************/
void TWRC_start(void)
{
	TWCR = (1<<TWINT)|(1<<TWSTA)|(1<<TWEN);
	while (!(TWCR & (1<<TWINT)));
}

/******************************************************************************
* Function:         	void TWRC_writebyte(uint8_t adat)
* Description:		
* Input:
* Output:
* Notes:
******************************************************************************/
void TWRC_writebyte(uint8_t adat)
{
	TWDR = adat;
	TWCR = (1<<TWINT) | (1<<TWEN);
	while (!(TWCR & (1<<TWINT)));
}

/******************************************************************************
* Function:         	uint8_t TWRC_readbyte(uint8_t ack)
* Description:		
* Input:
* Output:
* Notes:
******************************************************************************/
uint8_t TWRC_readbyte(uint8_t ack)
{
		if (ack)
		{
		  TWCR = (1<<TWINT) | (1<<TWEN)|(1<<TWEA);
		}
		else
		{
			TWCR = (1<<TWINT) | (1<<TWEN);
		}		
		
		while (!(TWCR & (1<<TWINT)));
		return(TWDR);
}

/******************************************************************************
* Function:         	void read_date(void)
* Description:		
* Input:
* Output:
* Notes:
******************************************************************************/
void read_date(void)
{
	uint8_t tmp;
		TWRC_start();
		
		TWRC_writebyte(SLA_W);
		TWRC_writebyte(0x00);
			 
		TWRC_start();
		
		TWRC_writebyte(SLA_R);
		date.day=TWRC_readbyte(1);	//day
		
		tmp=TWRC_readbyte(1);	//date
		date.date=(tmp>>4)*10 + (tmp & 0x0F);
		
		tmp=TWRC_readbyte(0);	//month
		date.month=(tmp>>4)*10 + (tmp & 0x0F);

		TWRC_stop();

		TWRC_start();
		
		TWRC_writebyte(SLA_W);
		TWRC_writebyte(0x06);
		
		TWRC_start();
		
		TWRC_writebyte(SLA_R);
		tmp=TWRC_readbyte(0);	//year
		date.year=(tmp>>4)*10 + (tmp & 0x0F);
		
		TWRC_stop();

}

/******************************************************************************
* Function:         	void set_date(date_t datum)
* Description:		
* Input:
* Output:
* Notes:
******************************************************************************/
void set_date_time(date_t datum)
{		
	
	TWRC_start();
	
	TWRC_writebyte(SLA_W);
	TWRC_writebyte(0x00);
	TWRC_writebyte(datum.sec);
	TWRC_writebyte(datum.min);
	TWRC_writebyte(datum.hrs);
	TWRC_writebyte(datum.day);
	TWRC_writebyte(datum.date);
	TWRC_writebyte(datum.month);
	TWRC_writebyte(datum.year);	
	
	TWRC_stop();	
}

/******************************************************************************
* Function:         	void TWRC_stop(void)
* Description:		
* Input:
* Output:
* Notes:
******************************************************************************/
void TWRC_stop(void)
{
	TWCR = (1<<TWINT)|(1<<TWEN)|(1<<TWSTO); 
}

/******************************************************************************
* Function:         	void TWRC_init(void)
* Description:		
* Input:
* Output:
* Notes:
******************************************************************************/
void TWRC_init(void)
{
	TWBR=0x08;	
}